package com.kasetoatz.hungryfrog;

import com.kasetoatz.hungryfrog.item.FrogInfestationItem;
import com.kasetoatz.hungryfrog.sensor.NearestBlockSensor;
import com.mojang.serialization.codecs.ListCodec;
import net.fabricmc.api.ModInitializer;
import net.fabricmc.fabric.api.itemgroup.v1.ItemGroupEvents;
import net.minecraft.entity.ai.brain.MemoryModuleType;
import net.minecraft.entity.ai.brain.sensor.SensorType;
import net.minecraft.item.Item;
import net.minecraft.item.ItemGroups;
import net.minecraft.item.Items;
import net.minecraft.registry.Registries;
import net.minecraft.registry.Registry;
import net.minecraft.registry.RegistryKey;
import net.minecraft.registry.RegistryKeys;
import net.minecraft.util.Identifier;
import net.minecraft.util.math.BlockPos;
import java.util.List;
import java.util.Optional;
import java.util.function.Function;

public class HungryFrog implements ModInitializer
{
    public static final MemoryModuleType<BlockPos> BLOCK_TO_EAT = new MemoryModuleType<>(Optional.of(BlockPos.CODEC));
    public static final MemoryModuleType<List<BlockPos>> UNREACHABLE_BLOCK_TARGETS = new MemoryModuleType<>(Optional.of(new ListCodec<>(BlockPos.CODEC, 0, 5)));
    public static final SensorType<NearestBlockSensor> NEAREST_BLOCK_SENSOR = new SensorType<>(NearestBlockSensor::new);
    public static final Item FROG_INFESTATION = register("frog_infestation", FrogInfestationItem::new, new Item.Settings().maxCount(1));

    public static Item register(String path, Function<Item.Settings, Item> factory, Item.Settings settings)
    {
        final RegistryKey<Item> registryKey = RegistryKey.of(RegistryKeys.ITEM, Identifier.of("hungryfrog", path));
        return Items.register(registryKey, factory, settings);
    }

    @Override
    public void onInitialize()
    {
        Registry.register(Registries.MEMORY_MODULE_TYPE, Identifier.of("hungryfrog", "block_to_eat"), BLOCK_TO_EAT);
        Registry.register(Registries.MEMORY_MODULE_TYPE, Identifier.of("hungryfrog", "unreachable_block_targets"), UNREACHABLE_BLOCK_TARGETS);
        Registry.register(Registries.SENSOR_TYPE, Identifier.of("hungryfrog", "nearest_block_sensor"), NEAREST_BLOCK_SENSOR);
        ItemGroupEvents.modifyEntriesEvent(ItemGroups.TOOLS).register(entries -> {
            entries.add(FROG_INFESTATION);
        });
    }
}
