package com.kasetoatz.hungryfrog.mixin;

import com.kasetoatz.hungryfrog.HungryFrog;
import net.minecraft.entity.LivingEntity;
import net.minecraft.entity.ai.brain.Brain;
import net.minecraft.entity.ai.brain.MemoryModuleType;
import net.minecraft.entity.ai.brain.sensor.Sensor;
import net.minecraft.entity.ai.brain.sensor.SensorType;
import net.minecraft.entity.attribute.DefaultAttributeContainer;
import net.minecraft.entity.attribute.EntityAttributes;
import net.minecraft.entity.passive.FrogEntity;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfoReturnable;
import java.util.ArrayList;

@Mixin(FrogEntity.class)
public class FrogEntityMixin
{
    @Inject(method = "isValidFrogFood", at = @At("HEAD"), cancellable = true)
    private static void isValidFrogFood(LivingEntity entity, CallbackInfoReturnable<Boolean> cir)
    {
        cir.setReturnValue(!(entity instanceof FrogEntity));
    }

    @Inject(method = "createFrogAttributes", at = @At("RETURN"), cancellable = true)
    private static void createFrogAttributes(CallbackInfoReturnable<DefaultAttributeContainer.Builder> cir)
    {
        DefaultAttributeContainer.Builder attr = cir.getReturnValue();
        attr.add(EntityAttributes.ATTACK_DAMAGE, Double.MAX_VALUE);
        cir.setReturnValue(attr);
    }

    @Inject(method = "createBrainProfile", at = @At("RETURN"), cancellable = true)
    private void createBrainProfile(CallbackInfoReturnable<Brain.Profile<FrogEntity>> cir)
    {
        @SuppressWarnings("unchecked")
        ProfileAccessor<FrogEntity> profile = (ProfileAccessor<FrogEntity>)(Object)cir.getReturnValue();
        if (profile != null)
        {
            ArrayList<MemoryModuleType<?>> memories = new ArrayList<>(profile.getMemoryModules());
            ArrayList<SensorType<? extends Sensor<? super FrogEntity>>> sensors = new ArrayList<>(profile.getSensors());
            memories.add(HungryFrog.BLOCK_TO_EAT);
            memories.add(HungryFrog.UNREACHABLE_BLOCK_TARGETS);
            sensors.add(HungryFrog.NEAREST_BLOCK_SENSOR);
            cir.setReturnValue(Brain.createProfile(memories, sensors));
        }
    }
}
