package com.kasetoatz.hungryfrog.sensor;

import com.google.common.collect.ImmutableSet;
import com.kasetoatz.hungryfrog.HungryFrog;
import net.minecraft.block.Blocks;
import net.minecraft.entity.ai.brain.Brain;
import net.minecraft.entity.ai.brain.MemoryModuleType;
import net.minecraft.entity.ai.brain.sensor.Sensor;
import net.minecraft.entity.attribute.EntityAttributes;
import net.minecraft.entity.passive.FrogEntity;
import net.minecraft.server.world.ServerWorld;
import net.minecraft.util.math.BlockPos;
import java.util.*;

public class NearestBlockSensor extends Sensor<FrogEntity>
{
    protected void sense(ServerWorld world, FrogEntity frog)
    {
        int range = (int) frog.getAttributeValue(EntityAttributes.FOLLOW_RANGE);
        Set<BlockPos> unreachable = new HashSet<>(frog.getBrain().getOptionalRegisteredMemory(HungryFrog.UNREACHABLE_BLOCK_TARGETS).orElse(new ArrayList<>()));
        BlockPos entityPos = frog.getBlockPos();
        BlockPos.Mutable pos = new BlockPos.Mutable();
        BlockPos closest = null;
        double closestDistance = Double.MAX_VALUE;
        for (int dx = -range; dx < range; dx++)
        {
            for (int dy = -range; dy < range; dy++)
            {
                for (int dz = -range; dz < range; dz++)
                {
                    pos.set(entityPos.getX() + dx, entityPos.getY() + dy, entityPos.getZ() + dz);
                    if (!unreachable.contains(pos) && !world.getBlockState(pos).isAir())
                    {
                        double distance = entityPos.getSquaredDistance(pos);
                        if (distance < closestDistance)
                        {
                            closestDistance = distance;
                            closest = pos.toImmutable();
                        }
                    }
                }
            }
        }
        Brain<?> brain = frog.getBrain();
        if (closest != null)
        {
            brain.remember(HungryFrog.BLOCK_TO_EAT, closest);
        }
        else
        {
            brain.forget(HungryFrog.BLOCK_TO_EAT);
        }
    }

    public Set<MemoryModuleType<?>> getOutputMemoryModules()
    {
        return ImmutableSet.of(HungryFrog.BLOCK_TO_EAT);
    }
}
